/*
 *  tcpclient
 *
 *  This is a very basic TCP client example to show the usage of
 *  sockets. 
 *  
 *  Copyright (C) 2010 Christian Pointner <equinox@realraum.at>
 *
 *  tcpclient is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  tcpclient is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with tcpclient. If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>

#include <arpa/inet.h>

int main(int argc, char** argv)
{
  if(argc <= 2) {
    fprintf(stderr,"too few arguments\n");
    exit(1);
  }
  printf("starting TCP Client\n");

  int sock = socket(AF_INET, SOCK_STREAM, 0);
  if(sock < 0) { 
    perror("socket() call failed");
    exit(-1);
  }
    
  struct sockaddr_in remote_addr;
  remote_addr.sin_family = AF_INET;
  remote_addr.sin_port = htons(atoi(argv[2]));
  if (inet_aton(argv[1], &remote_addr.sin_addr)==0) {
    perror("inet_aton() call failed");
    exit(-1);
  }

  if(connect(sock, (struct sockaddr *)&remote_addr, sizeof(remote_addr))==-1) { 
    perror("connect() call failed");
    exit(-1);
  }

  int len;
  uint8_t buffer[1500];
  for (;;) {
    if((len = read(0, buffer, 1500))==-1) {
      perror("read() call failed");
      exit(-1);
    }

    int offset = 0;
    int wlen;
    for(;;) {
      wlen = send(sock, &buffer[offset], len - offset, 0);
      if(wlen == -1) {
        if(errno != EINTR) {
          perror("write() call failed");
          exit(-1);
        }
        wlen = 0;
      }

      offset += wlen;
      if(offset+1 >= len)
        break;
    }
  }

  close(sock);
  
  printf("exiting TCP Client\n");

  return 0;
}
