/*
 *  tcpserver
 *
 *  This is a very basic TCP server example to show the usage of
 *  sockets. 
 *  
 *  Copyright (C) 2010 Christian Pointner <equinox@realraum.at>
 *
 *  tcpserver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  tcpserver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with tcpserver. If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include <sys/select.h>

int main(int argc, char** argv)
{
  printf("starting TCP Server\n");

  int server = socket(AF_INET, SOCK_STREAM, 0);
  if(server < 0) { 
    perror("socket() call failed");
    exit(-1);
  }
    
  struct sockaddr_in local_addr;
  memset((char *) &local_addr, 0, sizeof(local_addr));
  local_addr.sin_family = AF_INET;
  local_addr.sin_port = htons(1234);
  local_addr.sin_addr.s_addr = htonl(INADDR_ANY);

  if(bind(server, (struct sockaddr *)&local_addr, sizeof(local_addr))==-1) { 
    perror("bind() call failed");
    exit(-1);
  }

  if(listen(server, 5)==-1) { 
    perror("listen() call failed");
    exit(-1);
  }

  fd_set readfds, tmpfds;
  FD_ZERO(&readfds);
  FD_SET(server, &readfds);
  int max_fd = server;
  uint8_t buffer[1500];
  struct sockaddr_in remote_addr;
  memset (&remote_addr, 0, sizeof(remote_addr));
  int len, alen=sizeof(remote_addr);
  for (;;) {
    memcpy(&tmpfds, &readfds, sizeof(tmpfds));
    int num = select(max_fd+1, &tmpfds, NULL, NULL, NULL);
    if(num == -1 && errno != EINTR) {
      perror("select() call failed");
      exit(-1);
    }
    if(num == -1 || !num)
      continue;

    if(FD_ISSET(server, &tmpfds)) {
      int new_client = accept(server, (struct sockaddr *)&remote_addr, &alen);
      if(new_client==-1) {
        perror("accept() call failed");
        exit(-1);
      }
      printf("new connection %s:%d (fd=%d)\n", inet_ntoa(remote_addr.sin_addr), ntohs(remote_addr.sin_port), new_client);
      FD_SET(new_client, &readfds);
      max_fd = (max_fd < new_client) ? new_client : max_fd;
      FD_CLR(server, &tmpfds);
    }

    int fd;
    for(fd=0; fd<=max_fd; ++fd) {
      if(FD_ISSET(fd, &tmpfds)) {
        len = recv(fd, buffer, 1500, 0);
        if(len <= 0) {
          if(len < 0) perror("recv() call failed");
          else fprintf(stderr, "client closed connection\n");
          
          printf("removing client (fd=%d)\n", fd);
          FD_CLR(fd, &readfds);
          close(fd);
        }
        else
          printf("Received %d Bytes from client (fd=%d)\n", len, fd);
      }
    }
  }

  close(server);
  
  printf("exiting TCP Server\n");

  return 0;
}
